
#ifndef VTKPVVTKEXTENSIONSCORE_EXPORT_H
#define VTKPVVTKEXTENSIONSCORE_EXPORT_H

#ifdef VTKPVVTKEXTENSIONSCORE_STATIC_DEFINE
#  define VTKPVVTKEXTENSIONSCORE_EXPORT
#  define VTKPVVTKEXTENSIONSCORE_NO_EXPORT
#else
#  ifndef VTKPVVTKEXTENSIONSCORE_EXPORT
#    ifdef VTKExtensionsCore_EXPORTS
        /* We are building this library */
#      define VTKPVVTKEXTENSIONSCORE_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKPVVTKEXTENSIONSCORE_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKPVVTKEXTENSIONSCORE_NO_EXPORT
#    define VTKPVVTKEXTENSIONSCORE_NO_EXPORT 
#  endif
#endif

#ifndef VTKPVVTKEXTENSIONSCORE_DEPRECATED
#  define VTKPVVTKEXTENSIONSCORE_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKPVVTKEXTENSIONSCORE_DEPRECATED_EXPORT
#  define VTKPVVTKEXTENSIONSCORE_DEPRECATED_EXPORT VTKPVVTKEXTENSIONSCORE_EXPORT VTKPVVTKEXTENSIONSCORE_DEPRECATED
#endif

#ifndef VTKPVVTKEXTENSIONSCORE_DEPRECATED_NO_EXPORT
#  define VTKPVVTKEXTENSIONSCORE_DEPRECATED_NO_EXPORT VTKPVVTKEXTENSIONSCORE_NO_EXPORT VTKPVVTKEXTENSIONSCORE_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKPVVTKEXTENSIONSCORE_NO_DEPRECATED
#    define VTKPVVTKEXTENSIONSCORE_NO_DEPRECATED
#  endif
#endif

#endif /* VTKPVVTKEXTENSIONSCORE_EXPORT_H */
